<?php
/*
Plugin Name: bbP Signature
Plugin URI: http://arjunsk.in/bbp-signature/
Description: This plugin allows users to add a signature below their topics and replies.
Author: Arjun S Kumar
Version: 1.0
Author URI: http://arjunsk.in/
*/

/**
 * @todo add max length feature at admin panel.
 * @todo add support for BuddyPress forums
 *
*/
require_once 'stringparser_bbcode.class.php';



function bbp_user_signature( $user_id = 0 ) {
	echo bbp_get_user_signature( $user_id );
}

	function bbp_get_user_signature( $user_id = 0 ) {
		$signature = get_user_meta( $user_id, '_bbp_signature', true );
		return apply_filters( 'bbp_get_user_signature', $signature, $user_id );
	}
	
add_action( 'bbp_user_edit_after_about', 'bbp_edit_user_signature'  );	
add_filter( 'bbp_get_user_signature',    'wptexturize',        3    );
add_filter( 'bbp_get_user_signature',    'html_entity_decode', 7    );
add_filter( 'bbp_get_user_signature',    'convert_chars',      5    );
add_filter( 'bbp_get_user_signature',    'make_clickable',     9    );
add_filter( 'bbp_get_user_signature',    'capital_P_dangit',   10   );
add_filter( 'bbp_get_user_signature',    'force_balance_tags', 25   );
add_filter( 'bbp_get_user_signature',    'convert_smilies',    20   );
add_filter( 'bbp_get_user_signature',    'wpautop',            30   );
	
function bbp_edit_user_signature() {
	
	// Get the displayed users signature
	$signature = bbp_get_displayed_user_field( '_bbp_signature' ); ?>

		<div>
			<label for="_bbp_signature"><?php _e( 'Signature', 'bbpress' ); ?></label>
			<textarea name="_bbp_signature" id="_bbp_signature" rows="5" cols="30" onkeypress="return imposeMaxLength(this, 499);"><?php echo $signature; ?></textarea>
			<span class="description"><?php _e( 'This will be shown publicly below your topics and replies.', 'bbpress' ); ?></span>
		</div>
<?php
}	
add_action( 'wp_print_styles', 'bbp_signature_css' );
function bbp_signature_css( ) {
	wp_enqueue_style('bbp-signature', plugins_url('bbp-signature/bbp-signature.css'), false, '0.1', 'all');
}

add_action( 'wp_enqueue_scripts', 'bbp_signature_max_length_js' );
function bbp_signature_max_length_js( ) {
	?>
    <script language="javascript" type="text/javascript">
		<!--
		function imposeMaxLength(Object, MaxLen)
		{
  		return (Object.value.length <= MaxLen);
		}
		-->
	</script>
    <?php	
}

function bbp_reply_content_append_user_signature( $content = '', $reply_id = 0, $args = array() ) {
	// Default arguments
	$bbcode = new StringParser_BBCode ();
	$bbcode->addFilter (STRINGPARSER_FILTER_PRE, 'convertlinebreaks');

	$bbcode->addParser (array ('block', 'inline', 'link', 'listitem'), 'htmlspecialchars');
	$bbcode->addParser (array ('block', 'inline', 'link', 'listitem'), 'nl2br');
	$bbcode->addParser ('list', 'bbcode_stripcontents');

	$bbcode->addCode ('b', 'simple_replace', null, array ('start_tag' => '<b>', 'end_tag' => '</b>'),
			  'inline', array ('listitem', 'block', 'inline', 'link'), array ());
	$bbcode->addCode ('i', 'simple_replace', null, array ('start_tag' => '<i>', 'end_tag' => '</i>'),
			  'inline', array ('listitem', 'block', 'inline', 'link'), array ());
	$bbcode->addCode ('url', 'usecontent?', 'do_bbcode_url', array ('usecontent_param' => 'default'),
			  'link', array ('listitem', 'block', 'inline'), array ('link'));
	$bbcode->addCode ('link', 'callback_replace_single', 'do_bbcode_url', array (),
			  'link', array ('listitem', 'block', 'inline'), array ('link'));
	$bbcode->addCode ('img', 'usecontent', 'do_bbcode_img', array (),
			  'image', array ('listitem', 'block', 'inline', 'link'), array ());
	$bbcode->addCode ('bild', 'usecontent', 'do_bbcode_img', array (),
			  'image', array ('listitem', 'block', 'inline', 'link'), array ());
	$bbcode->setOccurrenceType ('img', 'image');
	$bbcode->setOccurrenceType ('bild', 'image');
	$bbcode->setMaxOccurrences ('image', 2);
	$bbcode->addCode ('list', 'simple_replace', null, array ('start_tag' => '<ul>', 'end_tag' => '</ul>'),
			  'list', array ('block', 'listitem'), array ());
	$bbcode->addCode ('*', 'simple_replace', null, array ('start_tag' => '<li>', 'end_tag' => '</li>'),
			  'listitem', array ('list'), array ());
	$bbcode->setCodeFlag ('*', 'closetag', BBCODE_CLOSETAG_OPTIONAL);
	$bbcode->setCodeFlag ('*', 'paragraphs', true);
	$bbcode->setCodeFlag ('list', 'paragraph_type', BBCODE_PARAGRAPH_BLOCK_ELEMENT);
	$bbcode->setCodeFlag ('list', 'opentag.before.newline', BBCODE_NEWLINE_DROP);
	$bbcode->setCodeFlag ('list', 'closetag.before.newline', BBCODE_NEWLINE_DROP);
	$bbcode->setRootParagraphHandling (true);
	$defaults = array(
		'separator' => '<hr />',
		'before'    => '<div class="bbp-reply-signature">',
		'after'     => '</div>'
	);
	$r = wp_parse_args( $args, $defaults );
	extract( $r );

	// Verify topic id, get author id, and potential signature
	$reply_id  = bbp_get_reply_id       ( $reply_id );
	$user_id   = bbp_get_reply_author_id( $reply_id );
	$thesig =  $bbcode->parse(xprofile_get_field_data( 'Signature', $user_id ));
	$signature = $thesig;
	//$signature = bbp_get_user_signature ( $user_id  );

	// If signature exists, adjust the content accordingly
	if ( !empty( $signature ) )
		$content = $content . $separator . $before . $signature . $after;

	return apply_filters( 'bbp_reply_content_append_signature', $content, $reply_id, $separator );
}

if ( !is_admin() ) {
	add_filter( 'bbp_get_reply_content', 'bbp_reply_content_append_user_signature', 1, 2 );
	add_filter( 'bbp_get_reply_content', 'bbp_reply_content_append_revisions',      1, 4 );
}

function bbp_topic_content_append_user_signature( $content = '', $topic_id = 0, $args = array() ) {
	$bbcode = new StringParser_BBCode ();
	$bbcode->addFilter (STRINGPARSER_FILTER_PRE, 'convertlinebreaks');

	$bbcode->addParser (array ('block', 'inline', 'link', 'listitem'), 'htmlspecialchars');
	$bbcode->addParser (array ('block', 'inline', 'link', 'listitem'), 'nl2br');
	$bbcode->addParser ('list', 'bbcode_stripcontents');

	$bbcode->addCode ('b', 'simple_replace', null, array ('start_tag' => '<b>', 'end_tag' => '</b>'),
			  'inline', array ('listitem', 'block', 'inline', 'link'), array ());
	$bbcode->addCode ('i', 'simple_replace', null, array ('start_tag' => '<i>', 'end_tag' => '</i>'),
			  'inline', array ('listitem', 'block', 'inline', 'link'), array ());
	$bbcode->addCode ('url', 'usecontent?', 'do_bbcode_url', array ('usecontent_param' => 'default'),
			  'link', array ('listitem', 'block', 'inline'), array ('link'));
	$bbcode->addCode ('link', 'callback_replace_single', 'do_bbcode_url', array (),
			  'link', array ('listitem', 'block', 'inline'), array ('link'));
	$bbcode->addCode ('img', 'usecontent', 'do_bbcode_img', array (),
			  'image', array ('listitem', 'block', 'inline', 'link'), array ());
	$bbcode->addCode ('bild', 'usecontent', 'do_bbcode_img', array (),
			  'image', array ('listitem', 'block', 'inline', 'link'), array ());
	$bbcode->setOccurrenceType ('img', 'image');
	$bbcode->setOccurrenceType ('bild', 'image');
	$bbcode->setMaxOccurrences ('image', 2);
	$bbcode->addCode ('list', 'simple_replace', null, array ('start_tag' => '<ul>', 'end_tag' => '</ul>'),
			  'list', array ('block', 'listitem'), array ());
	$bbcode->addCode ('*', 'simple_replace', null, array ('start_tag' => '<li>', 'end_tag' => '</li>'),
			  'listitem', array ('list'), array ());
	$bbcode->setCodeFlag ('*', 'closetag', BBCODE_CLOSETAG_OPTIONAL);
	$bbcode->setCodeFlag ('*', 'paragraphs', true);
	$bbcode->setCodeFlag ('list', 'paragraph_type', BBCODE_PARAGRAPH_BLOCK_ELEMENT);
	$bbcode->setCodeFlag ('list', 'opentag.before.newline', BBCODE_NEWLINE_DROP);
	$bbcode->setCodeFlag ('list', 'closetag.before.newline', BBCODE_NEWLINE_DROP);
	$bbcode->setRootParagraphHandling (true);
	// Default arguments
	$defaults = array(
		'separator' => '<hr />',
		'before'    => '<div class="bbp-topic-signature">',
		'after'     => '</div>'
	);
	$r = wp_parse_args( $args, $defaults );
	extract( $r );

	// Verify topic id, get author id, and potential signature
	$topic_id  = bbp_get_topic_id       ( $topic_id );
	$user_id   = bbp_get_topic_author_id( $topic_id );
	$thesig =  $bbcode->parse(xprofile_get_field_data( 'Signature', $user_id ));
	$signature = $thesig;

	// If signature exists, adjust the content accordingly
	if ( !empty( $signature ) )
		$content = $content . $separator . $before . $signature . $after;

	return apply_filters( 'bbp_topic_content_append_signature', $content, $topic_id, $separator );
}

if ( !is_admin() ) {
	add_filter( 'bbp_get_topic_content', 'bbp_topic_content_append_user_signature', 1, 2 );
	add_filter( 'bbp_get_topic_content', 'bbp_topic_content_append_revisions',      1, 4 );
}

function bbp_edit_user_signature_handler( $user_id ) {

	// Sanitize the user signature
	$signature = !empty( $_POST['_bbp_signature'] ) ? $_POST['_bbp_signature'] : '';
	$signature = apply_filters( 'bbp_edit_user_signature_handler', $signature );

	// Update signature user meta
	if ( !empty( $signature ) )
		update_user_meta( $user_id, '_bbp_signature', $signature );

	// Delete signature user meta
	else
		delete_user_meta( $user_id, '_bbp_signature' );
}
add_action( 'personal_options_update',         'bbp_edit_user_signature_handler' );
add_action( 'edit_user_profile_update',        'bbp_edit_user_signature_handler' );
add_filter( 'bbp_edit_user_signature_handler', 'trim'                            );
add_filter( 'bbp_edit_user_signature_handler', 'wp_filter_kses'                  );
add_filter( 'bbp_edit_user_signature_handler', 'force_balance_tags'              );
add_filter( 'bbp_edit_user_signature_handler', '_wp_specialchars'                );

function convertlinebreaks ($text) {
    return preg_replace ("/\015\012|\015|\012/", "\n", $text);
}

// Remove everything but the newline charachter
function bbcode_stripcontents ($text) {
    return preg_replace ("/[^\n]/", '', $text);
}

function do_bbcode_url ($action, $attributes, $content, $params, $node_object) {
    if (!isset ($attributes['default'])) {
        $url = $content;
        $text = htmlspecialchars ($content);
    } else {
        $url = $attributes['default'];
        $text = $content;
    }
    if ($action == 'validate') {
        if (substr ($url, 0, 5) == 'data:' || substr ($url, 0, 5) == 'file:'
          || substr ($url, 0, 11) == 'javascript:' || substr ($url, 0, 4) == 'jar:') {
            return false;
        }
        return true;
    }
    return '<a href="'.htmlspecialchars ($url).'">'.$text.'</a>';
}

// Function to include images
function do_bbcode_img ($action, $attributes, $content, $params, $node_object) {
    if ($action == 'validate') {
        if (substr ($content, 0, 5) == 'data:' || substr ($content, 0, 5) == 'file:'
          || substr ($content, 0, 11) == 'javascript:' || substr ($content, 0, 4) == 'jar:') {
            return false;
        }
        return true;
    }
    return '<img src="'.htmlspecialchars($content).'" alt="">';
}	
?>